<?php
/*
 * Copyright (c) 2014 Mathieu Parent <sathieu@debian.org>
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 */

namespace Pkgtools\Phppearchannel;

/**
* This class parses channel.xml
*
* @copyright Copyright (c) 2014 Mathieu Parent <sathieu@debian.org>
* @author Mathieu Parent <sathieu@debian.org>
* @license Expat http://www.jclark.com/xml/copying.txt
*/
class Source {
    /**
     * channel.xml file path
     *
     * @var string
     */
    protected $_path = NULL;

    /**
     * SimpleXML
     *
     * @var SimpleXMLElement
     */
    protected $_xml = NULL;

    /**
     * Constructor
     *
     * @param string $filename
     */
    function __construct($dir_name) {
        // Find channel.xml
        $dir_name = realpath($dir_name);
        foreach (Array('channel.xml') as $try) {
            if (is_file("$dir_name/$try")) {
                $this->_path = "$dir_name/$try";
            }
        }
        if (is_null($this->_path)) {
            throw new \InvalidArgumentException('channel.xml not found');
        }
        // Parse XML
        $use_errors = libxml_use_internal_errors(true);
        $this->_xml = simplexml_load_file($this->_path);
        /*
        foreach (libxml_get_errors() as $error) {
            var_dump($error);
        }
        // */
        libxml_clear_errors();
        libxml_use_internal_errors($use_errors);
        if ($this->_xml ===  FALSE) {
            throw new \InvalidArgumentException('Error parsing channel.xml');
        }
    }

    /**
     * Raw properties getter
     *
     * @param string $property
     */
    function __get($property) {
        switch($property) {
            case 'name':
            case 'summary':
            case 'suggestedalias':
                return $this->_xml->{$property};
            default:
                throw new \InvalidArgumentException("Unknown property: '$property'");
        }
    }
}
